/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.example;

import java.util.Collection;
import java.util.Iterator;

import org.simantics.databoard.DataTypes;
import org.simantics.databoard.Datasources;
import org.simantics.databoard.accessor.Accessor;
import org.simantics.databoard.accessor.Accessor.Listener;
import org.simantics.databoard.accessor.DoubleAccessor;
import org.simantics.databoard.accessor.RecordAccessor;
import org.simantics.databoard.accessor.event.Event;
import org.simantics.databoard.accessor.interestset.InterestSet;
import org.simantics.databoard.accessor.interestset.MapInterestSet;
import org.simantics.databoard.accessor.reference.AccessorReference;
import org.simantics.databoard.accessor.reference.FieldNameReference;
import org.simantics.databoard.binding.mutable.MutableVariant;
import org.simantics.databoard.binding.mutable.Variant;
import org.simantics.databoard.datasource.ExampleDatasource;
import org.simantics.databoard.datasource.StepwiseDatasourceAccessor;

public class DatasourceExample {

	public static void main(String[] args) throws Exception {
		
		// Create datasource. See ExampleDatasource
		ExampleDatasource datasource = new ExampleDatasource();
		
		// Open accessor to datasource
		StepwiseDatasourceAccessor accessor = Datasources.getAccessor(datasource);
		Accessor nodes = accessor.getAccessor( new FieldNameReference("nodes") );
		
		// Get root id constant
		Variant rootId = Datasources.ROOT_ID;
		
		// Get children of root
		Variant children[] = Datasources.getChildren(accessor, rootId);
		
		// Print children, and grand children
		for (Variant nodeId : children) {
			System.out.println( nodeId );
			Variant children2[] = Datasources.getChildren(accessor, nodeId);
			for (Variant nodeId2 : children2)
				System.out.println("  o " + nodeId2 );
		}
		
		// Open accessor to Sine variable
		RecordAccessor sine = accessor.getAccessor( AccessorReference.parseString("nodes/SSINE") );
		// Open accessor to the value
		DoubleAccessor sineValue = accessor.getAccessor( AccessorReference.parseString("nodes/SSINE/value/o/v") );
		DoubleAccessor rampValue = accessor.getAccessor( AccessorReference.parseString("nodes/SRAMP/value/o/v") );
		DoubleAccessor timeValue = accessor.getAccessor( AccessorReference.parseString("nodes/STIME/value/o/v") );
		
		Listener listener = new Listener() {
			@Override
			public void onEvents(Collection<Event> events) {
				System.out.println("Notification: "+events.size()+" events");
				Iterator<Event> iter = events.iterator();
				while (iter.hasNext()) {
					System.out.println("  "+iter.next());
				}
			}};
		
		Variant keySine = Variant.ofInstance("SINE");
		Variant keyRamp = Variant.ofInstance("RAMP");
		Variant keyTime = Variant.ofInstance("TIME");
		Variant keyVector = Variant.ofInstance("VECTOR");

		// Create interest for SINE, RAMP and TIME
		InterestSet nodeInterest = InterestSet.newInterestSet( DataTypes.getDataType("Node"), true, true, true); 
		MapInterestSet is = new MapInterestSet();
		is.addCompositeInterest(keySine, nodeInterest);
		is.addCompositeInterest(keyRamp, nodeInterest);
		is.addCompositeInterest(keyTime, nodeInterest);
		is.addCompositeInterest(keyVector, nodeInterest);
		
		nodes.addListener(listener, is, null);
		
		// BUG, Oh fuck
		// TODO It should sample those with accessors 
		// And those with listeners
		System.gc();
		
		// Step and read value 10 times
		for (int i=0; i<10; i++) {
//			double y = sineValue.getValue();
			System.out.println("Step "+i);
			datasource.step();
		}

		// Modify with accessor
		timeValue.setValue(0.0);		
		
		for (int i=0; i<10; i++) {
//			double y = sineValue.getValue();
			System.out.println("Step "+(i+10));
			datasource.step();
		}

		// Makde modification with-in datasource
		datasource.setValue(keyTime, Variant.ofInstance(0.0));
		System.gc();

		for (int i=0; i<10; i++) {
			System.out.println("Step "+(i+20));
			datasource.step();
		}
		
		nodes.removeListener(listener);
		accessor.dispose();
	}
	
}

