/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.example.old;

import java.awt.geom.Rectangle2D;
import java.nio.BufferUnderflowException;

import org.simantics.databoard.Bindings;
import org.simantics.databoard.DataTypes;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.serialization.BinarySerializationFormat;
import org.simantics.databoard.serialization.SerializationException;
import org.simantics.databoard.serialization.Serializer;
import org.simantics.databoard.type.DataType;

public class ValidatorExample {

	public static final DataType typeType = DataTypes.getDataTypeUnchecked( DataType.class );
	public static final Binding typeBinding = Bindings.getBindingUnchecked( DataType.class );
	public static final Serializer typeSerializer = typeBinding.serializerUnchecked( BinarySerializationFormat.INSTANCE );
	public static final Serializer typeValidator = Bindings.getUnboundBinding( typeType ).serializerUnchecked( BinarySerializationFormat.INSTANCE );

	public static final DataType rectangleType = DataTypes.getDataTypeUnchecked( Rectangle2D.Double.class );
	public static final Binding rectangleBinding = Bindings.getBindingUnchecked( Rectangle2D.Double.class );
	public static final Serializer rectangleSerializer = rectangleBinding.serializerUnchecked( BinarySerializationFormat.INSTANCE );
	public static final Serializer rectangleValidator = Bindings.getUnboundBinding( rectangleType ).serializerUnchecked( BinarySerializationFormat.INSTANCE );
	
	public static void main(String[] args) 
	throws Exception {
		// Create Rectangle
		Rectangle2D rect = new Rectangle2D.Double(20,30,90,100);
		// Serialize rectangle
		byte[] rectData = rectangleSerializer.serialize( rect );
		// Validate rectangle data
		rectangleValidator.deserialize( rectData );

		// Create Rectangle
		DataType type = rectangleType;
		// Serialize rectangle type
		byte[] typeData = typeSerializer.serialize( type );
		// Validate type data
		try {
			typeValidator.deserialize( typeData );
			System.out.println("The data is valid");
		} catch (SerializationException invalidData) {			
			System.out.println("The data not valid");
		} catch (BufferUnderflowException invalidData) {
			System.out.println("Not enough data");
		}
	}
	
}

