/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.accessor.reference;


/**
 * "f-<number>"
 *
 * @author Toni Kalajainen <toni.kalajainen@vtt.fi>
 */
public class FieldIndexReference extends AccessorReference {

	int index;
	
	public FieldIndexReference(int index) {
		this.index = index;
	}
	
	public FieldIndexReference(int index, AccessorReference child) {
		super(child);
		this.index = index;
	}
	
	public int getIndex() {
		return index;
	}
	
	@Override
	public String toString(boolean labelReference) {
		return labelReference ? ""+index : "f-"+index;
	}

	@Override
	public AccessorReference clone() {
		return new FieldIndexReference(index, childReference==null ? null : childReference.clone());
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof FieldIndexReference == false) return false;
		FieldIndexReference other = (FieldIndexReference) obj;
		if (other.index != index) return false;
		if (other.hasChildReference() != hasChildReference()) return false;
		if (hasChildReference() && !other.childReference.equals(childReference)) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int hash = 466545 + 37 * index;
		if (hasChildReference()) hash = 31*hash + childReference.hashCode();		
		return hash;
	}

	
}

