/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.binding.java;

import java.util.Iterator;
import java.util.Set;

import org.simantics.databoard.binding.ArrayBinding;
import org.simantics.databoard.binding.BooleanBinding;
import org.simantics.databoard.binding.error.BindingException;
import org.simantics.databoard.binding.error.UnsupportedOperationException;
import org.simantics.databoard.util.IdentityPair;

/**
 * Binds the type Boolean[]-type to boolean[] Class.
 */
public final class BooleanArrayBinding extends ArrayBinding {

	// Binding for variable width boolean array
	@Deprecated
	public static final BooleanArrayBinding INSTANCE = new BooleanArrayBinding();
	
	public BooleanArrayBinding() {
		super(BooleanBindingDefault.INSTANCE);
	}
	
	public BooleanArrayBinding(BooleanBinding componentBinding) {
		super(componentBinding);
	}
	
	@Override
	public Object create() {
		return new boolean[0];
	}
	
	@Override
	public Object create(int length, Iterator<Object> values) {
		boolean[] array = new boolean[length];
		for(int i=0;i<length;++i)
			array[i] = (Boolean)values.next();
		return array;
	}
	
	public Object create(Object...values) {
		boolean[] result = new boolean[values.length];
		for (int i=0; i<values.length; i++)
			result[i] = (Boolean) values[i];
		return result;
	}	

	/**
	 * Create an array object.
	 * Note! The argument is consumed (is used in the result).
	 * 
	 * @param array
	 * @return an object that contains the array
	 */
	public Object create(boolean[] array) {
		return array;
	}
		
	@Override
	public Object get(Object array, int index) throws BindingException {
		if (!isInstance(array)) throw new BindingException("Unexpected class "+array.getClass().getSimpleName()+", boolean[] expected");
		return ((boolean[])array)[index];
	}

	@Override
	public void getAll(Object array, Object[] result) throws BindingException {
		boolean[] a = (boolean[])array;
		for (int i=0; i<a.length; i++)
			result[i] = a[i];
	}
	
	@Override
	public void set(Object array, int index, Object value)
			throws BindingException {
		boolean[] a = (boolean[])array;
		a[index] = (Boolean) value;
	}
	
	@Override
	public int size(Object array) throws BindingException {
		if (!isInstance(array)) throw new BindingException("Unexpected class "+array.getClass().getSimpleName()+", boolean[] expected");
		return ((boolean[])array).length;
	}

	@Override
	public boolean isInstance(Object obj) {
		return obj instanceof boolean[];
	}
	
	@Override
	public boolean isImmutable() {
		return true;
	}

	public boolean[] getArray(Object array) throws BindingException {
		if (!isInstance(array)) throw new BindingException("Unexpected class "+array.getClass().getSimpleName()+", boolean[] expected");
		return (boolean[]) array;
	}	
	
    @Override
    public int deepCompare(Object o1, Object o2,
    		Set<IdentityPair<Object, Object>> compareHistory)
    		throws BindingException {
    	boolean[] a1 = (boolean[]) o1;
    	boolean[] a2 = (boolean[]) o2;
		// Compare Lengths
		int l1 = a1.length;
		int l2 = a2.length;
		int dif = l1 - l2;
		if (dif!=0) return dif;
		// Compare elements
		for (int i=0; i<l1; i++) {
			boolean v1 = a1[i];
			boolean v2 = a2[i];
			dif = (v2 == v1 ? 0 : (v1 ? 1 : -1));
			if (dif!=0) return dif;
		}
		return 0;
    }    
    
    @Override
    public void add(Object array, int index, Object element)
    		throws BindingException, IndexOutOfBoundsException {
    	throw new UnsupportedOperationException();
    }
    
	@Override
	public void remove(Object array, int index, int count) throws BindingException {
    	throw new UnsupportedOperationException();
	}
    
	
}
