/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import java.util.Set;

import org.simantics.databoard.util.IdentityPair;
import org.simantics.databoard.util.ObjectUtils;

public class MapType extends DataType {

	DataType keyType;
	DataType valueType;
	
	public MapType() {		
	}
	
	public MapType(DataType keyType, DataType valueType) {
		if (keyType==null || valueType==null)
			throw new IllegalArgumentException("Argument must not be null");
		this.keyType = keyType;
		this.valueType = valueType;
	}
	
	@Override
	public void accept(Visitor1 v, Object obj) {
	    v.visit(this, obj);        
	}

	@Override
	public <T> T accept(Visitor<T> v) {
	    return v.visit(this);
	}

	@Override
	protected boolean deepEquals(Object obj,
			Set<IdentityPair<DataType, DataType>> compareHistory) {
		
		if (this==obj) return true;
		if (obj instanceof MapType == false) return false;
		MapType other = (MapType) obj;
		
		return keyType.deepEquals(other.keyType, compareHistory) && 
			valueType.deepEquals(other.valueType, compareHistory);
	}
	
	@Override
	protected void collectSubtypes(Set<DataType> subtypes, Set<DataType> recursiveSubtypes) {
		keyType.collectSubtypes(subtypes, recursiveSubtypes);
		valueType.collectSubtypes(subtypes, recursiveSubtypes);
	}
	
	@Override
	public int hashCode() {
		if (keyType==this) return 0;
		if (valueType==this) return 0;
		return 0x232ae +
			ObjectUtils.hashCode(keyType) * 13 +
			ObjectUtils.hashCode(valueType) * 17;
	}
	
	public DataType getKeyType() {
		return keyType;
	}
	
	public DataType getValueType() {
		return valueType;
	}
	
	public void setKeyType(DataType keyType) {
		this.keyType = keyType;
	}
	
	public void setValueType(DataType valueType) {
		this.valueType = valueType;
	}

}

