/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.util.binary;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * BinaryReadable is a readable stream of bytes.
 * 
 * Common interface for ByteBuffer, byte[], InputStream, RandomAccessFile.
 * 
 * @see ByteBufferReadable
 * @see FileReadable
 * @see InputStreamReadable
 * @author Toni Kalajainen (toni.kalajainen@vtt.fi)
 */
public interface BinaryReadable {

    byte get()
    throws IOException, IndexOutOfBoundsException;

    void get(byte[] dst, int offset, int length)
    throws IOException, IndexOutOfBoundsException;
    
    void get(byte[] dst)
    throws IOException, IndexOutOfBoundsException;
    
    /**
     * Get buf fully 
     * 
     * @param buf
     * @throws IOException
     */
    void get(ByteBuffer buf)
    throws IOException, IndexOutOfBoundsException;
    
    /**
     * Get fully length bytes
     * 
     * @param buf
     * @param length
     * @throws IOException
     */
    void get(ByteBuffer buf, int length)
    throws IOException, IndexOutOfBoundsException;
    
    short getShort()
    throws IOException, IndexOutOfBoundsException;
    
    int getInt()
    throws IOException, IndexOutOfBoundsException;
    
    long getLong()
    throws IOException, IndexOutOfBoundsException;
    
    float getFloat()
    throws IOException, IndexOutOfBoundsException;
    
    double getDouble()
    throws IOException, IndexOutOfBoundsException;
    
	long length() throws IOException;
	
	ByteOrder order();

	void order(ByteOrder order);
    
	long position() throws IOException;

	void skip(long bytes) throws IOException;
	
}

